package gltest;

import java.awt.Color;
import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.GLJPanel;
import javax.media.opengl.glu.GLU;



/**
 * OpenGL renderer implemented with use of a GLJPanel
 */
public class OpenGLRenderer3D {

	private GLU glu_ = new GLU();
	private Color back_color;
	private Transform3D transform3D;
	private int bufwidth, bufheight;
	private int displayListID;
	private Viewer viewer;
	protected GLJPanel drawComponent;

	/**
	 * Creates the GLJPanel and its associated GLEventListener and returns it to the Viewer class 
	 */
	public GLJPanel getRenderComponent(Viewer v) {
		System.out.println("start render with out pbuff");
		viewer = v;
		if (drawComponent == null) { 
		drawComponent = new GLJPanel();
		drawComponent.addGLEventListener( new GLEventListener()  {
			public void display(GLAutoDrawable arg0) {
				viewer.prepare(drawComponent.getWidth(),drawComponent.getHeight());
				GL gl = arg0.getGL();
				draw(gl);
			}
			public void displayChanged(GLAutoDrawable arg0, boolean arg1,
					boolean arg2) {
			}
			public void init(GLAutoDrawable arg0) {  
				System.out.println("init");
				displayListID = arg0.getGL().glGenLists(1); System.out.println(" ID = " + displayListID);
				viewer.setNeedsRedraw(true);
			}
			public void reshape(GLAutoDrawable arg0, int arg1, int arg2,
					int arg3, int arg4) {
			}
		});
		}
		return drawComponent;
	}

	/**
	 * Performs all the OpenGL setup and sets up using display lists.
	 * @param transform
	 * @param width
	 * @param height
	 */
	public void startRender( Transform3D transform, int width, int height) { 
		
		transform3D = transform;

		bufwidth = width;
		bufheight = height;

        if(drawComponent == null){System.out.println("drawcomponent null");}
        System.out.println("drawcomponent");
		GL gl = drawComponent.getGL();	

		gl.glMatrixMode(GL.GL_MODELVIEW);
		gl.glLoadIdentity();

		back_color = Color.WHITE;

		Color amb = new Color(0.2F,0.2F,0.2F);
		Color color = new Color(0.25F, 0.25F, 0.2F);
		Color c_dir1 = Color.RED;
		Color c_dir2 = Color.GREEN;
		Color c_dir3 = Color.BLUE;
		Vector3D dir1 = new Vector3D(-0.707,0,-0.707);
		Vector3D dir2 = new Vector3D(-0.577,-0.577,-0.577);
		Vector3D dir3 = new Vector3D(0,-0.707,-0.707);


		float light_pos[] = { 0.0f, 0.0f, -5.0f,0 };
		float light_color[] = {(float)(color.getRed()/255.0), (float)(color.getGreen()/255.0),(float)(color.getBlue()/255.0),1 };			
		float amb_color[] = {(float)(amb.getRed()/255.0), (float)(amb.getGreen()/255.0), (float)(amb.getBlue()/255.0),1 };

		float light_dir[] = { (float)dir1.x, (float)dir1.y, (float)dir1.z,0 };
		float light_color1[] = {(float)(c_dir1.getRed()/255.0), (float)(c_dir1.getGreen()/255.0),(float)(c_dir1.getBlue()/255.0),1 };

		float light_dir2[] = { (float)dir2.x, (float)dir2.y, (float)dir2.z,0 };
		float light_color2[] = {(float)(c_dir2.getRed()/255.0), (float)(c_dir2.getGreen()/255.0),(float)(c_dir2.getBlue()/255.0),1 };

		float light_dir3[] = { (float)dir3.x, (float)dir3.y, (float)dir3.z,0 };
		float light_color3[] = {(float)(c_dir3.getRed()/255.0), (float)(c_dir3.getGreen()/255.0),(float)(c_dir3.getBlue()/255.0),1 };

		/* set up depth-buffering */
		gl.glEnable(GL.GL_DEPTH_TEST);

		gl.glLightModeli(GL.GL_LIGHT_MODEL_TWO_SIDE, GL.GL_TRUE);

		gl.glEnable(GL.GL_LIGHTING);
		gl.glLightModelfv(GL.GL_LIGHT_MODEL_AMBIENT,amb_color,0);

		gl.glFrontFace(GL.GL_CW);

		/* lighting set up */
		gl.glEnable(GL.GL_LIGHT0);
		gl.glEnable(GL.GL_LIGHT1);
		gl.glEnable(GL.GL_LIGHT2);
		gl.glEnable(GL.GL_LIGHT3);

		gl.glLightfv(GL.GL_LIGHT0,GL.GL_POSITION,light_pos,0);
		gl.glLightfv(GL.GL_LIGHT0,GL.GL_DIFFUSE,light_color,0);
		gl.glLightfv(GL.GL_LIGHT0,GL.GL_SPECULAR,light_color,0);

		gl.glLightfv(GL.GL_LIGHT1,GL.GL_POSITION,light_dir,0);
		gl.glLightfv(GL.GL_LIGHT1,GL.GL_DIFFUSE,light_color1,0);
		gl.glLightfv(GL.GL_LIGHT1,GL.GL_SPECULAR,light_color1,0);

		gl.glLightfv(GL.GL_LIGHT2,GL.GL_POSITION,light_dir2,0);
		gl.glLightfv(GL.GL_LIGHT2,GL.GL_DIFFUSE,light_color2,0);
		gl.glLightfv(GL.GL_LIGHT2,GL.GL_SPECULAR,light_color2,0);

		gl.glLightfv(GL.GL_LIGHT3,GL.GL_POSITION,light_dir3,0);
		gl.glLightfv(GL.GL_LIGHT3,GL.GL_DIFFUSE,light_color3,0);
		gl.glLightfv(GL.GL_LIGHT3,GL.GL_SPECULAR,light_color3,0);

		gl.glNewList(displayListID, GL.GL_COMPILE);
	}


    /**
     * Called at the end of the drawing commands to end the display list.
     *
     */
	public void endRender() {  
		GL gl = drawComponent.getGL();
		gl.glEndList();

	}

	/**
	 * Draws the contents of the display list to the screen.
	 */
	public void draw(GL gl) {

		gl.glViewport(0, 0, bufwidth, bufheight);

		transform3D.setAspectRatio(bufwidth, bufheight);

		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();

		double factor = transform3D.getClipDistance() / transform3D.getViewPoint().norm();
		gl.glFrustum(factor * transform3D.getXmin(), factor * transform3D.getXmax(), factor * transform3D.getYmin(), factor * transform3D.getYmax(), transform3D.getClipDistance(), 2*transform3D.getViewPoint().norm());

		/* define the viewing transformation */
		gl.glMatrixMode(GL.GL_MODELVIEW);
		gl.glLoadIdentity();
		double x = transform3D.getViewPoint().x;
		double y = transform3D.getViewPoint().y;
		double z = transform3D.getViewPoint().z;
		double look_x = transform3D.getViewDirection().x;
		double look_y = transform3D.getViewDirection().y;
		double look_z = transform3D.getViewDirection().z;
		double up_x = transform3D.getViewUp().x;
		double up_y = transform3D.getViewUp().y;
		double up_z = transform3D.getViewUp().z;
		glu_.gluLookAt(x,y,z,look_x,look_y,look_z,up_x,up_y,up_z);

		gl.glClearColor((float)(back_color.getRed()/255.0), (float)(back_color.getGreen()/255.0), (float)(back_color.getBlue()/255.0), 0.0f);
		gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

		gl.glCallList(displayListID);

		gl.glFlush();
	}

	/**
	 * Calls the necessary draw method.
	 */
	public void drawSurface(Grid3D surfaceData, boolean wireframe, boolean showGridOnSurface) {
		GL gl = drawComponent.getGL();

		long start = System.currentTimeMillis();

		if (wireframe) {
			drawWireFrame(gl,surfaceData);
		}
		else  {
			drawPolygons(gl, surfaceData, showGridOnSurface);
		}

		System.out.println( "Render time = " + (System.currentTimeMillis() - start));
	}

	/**
	 * Draws the surface and grid lines if turned on.
	 */
	private void drawPolygons(GL gl, Grid3D surfaceData, boolean showGrid) {


		Vector3D[][] points = surfaceData.getPointsOnSurface();

		float specular[] = {0.0f,0.0f,0.0f,1f};		
		gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_SPECULAR,specular,0);
		gl.glMaterialf(GL.GL_FRONT_AND_BACK,GL.GL_SHININESS,175);


		float[] c = { 0.8F, 0.8F, 0.8F, 1 };
		gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE, c, 0);

		//draw surface using quad strips
		for(int i=0;i<surfaceData.getVCount()-1;i++){  

			gl.glBegin(GL.GL_QUAD_STRIP);
			for(int l=0;l<surfaceData.getUCount();l++){

				Vector3D v1 = points[l][i+1];
				Vector3D n1 = surfaceData.getUnitNormal(l, i+1);
				gl.glNormal3d(n1.x, n1.y, n1.z);
				gl.glVertex3d(v1.x, v1.y, v1.z);

				Vector3D v = points[l][i];
				Vector3D n = surfaceData.getUnitNormal(l, i);
				gl.glNormal3d(n.x, n.y, n.z);
				gl.glVertex3d(v.x, v.y, v.z);


			}
			gl.glEnd();
		}

		if (showGrid) {//draw grid lines if showGrid is true
			
			gl.glMatrixMode(GL.GL_PROJECTION);
			gl.glPushMatrix();
			gl.glTranslated(0, 0, .02);
			gl.glDisable(GL.GL_LIGHTING);
	
	
			gl.glColor3f(0, 0, 0);
	
			//draw vertical lines
			for(int i = 0; i < surfaceData.getUCount(); i+=6){
				gl.glBegin(GL.GL_LINE_STRIP);
				for(int j=0;j<surfaceData.getVCount();j++){
					Vector3D v1 = points[i][j];
					gl.glVertex3d(v1.x, v1.y, v1.z);
				}
				gl.glEnd();
			}
			gl.glBegin(GL.GL_LINE_STRIP);
			for(int j=0;j<surfaceData.getVCount();j++){
				Vector3D v1 = points[surfaceData.getUCount()-1][j];
				gl.glVertex3d(v1.x, v1.y, v1.z);
			}
			gl.glEnd();
	
			//draw horizontal lines
			for(int i = 0; i < surfaceData.getVCount(); i+=6){
				gl.glBegin(GL.GL_LINE_STRIP);
				for(int j=0;j<surfaceData.getUCount();j++){
					Vector3D v1 = points[j][i];
					gl.glVertex3d(v1.x, v1.y, v1.z);
				}
				gl.glEnd();
			}
			gl.glBegin(GL.GL_LINE_STRIP);
			for(int j=0;j<surfaceData.getUCount();j++){
				Vector3D v1 = points[j][surfaceData.getVCount()-1];
				gl.glVertex3d(v1.x, v1.y, v1.z);
			}
			gl.glEnd();
	
			gl.glEnable(GL.GL_LIGHTING);
	
			gl.glPopMatrix();
			
		}

	}



	/**
	 * Draws only the wireframe surface
	 */
	private void drawWireFrame(GL gl, Grid3D surfaceData){

		Vector3D[][] points = surfaceData.getPointsOnSurface();


		gl.glDisable(GL.GL_LIGHTING);

		gl.glColor3f(0, 0, 0);

		//draw vertical lines
		for(int i = 0; i < surfaceData.getUCount(); i+=6){
			gl.glBegin(GL.GL_LINE_STRIP);
			for(int j=0;j<surfaceData.getVCount();j++){
				Vector3D v1 = points[i][j];
				gl.glVertex3d(v1.x, v1.y, v1.z);
			}
			gl.glEnd();
		}
		gl.glBegin(GL.GL_LINE_STRIP);
		for(int j=0;j<surfaceData.getVCount();j++){
			Vector3D v1 = points[surfaceData.getUCount()-1][j];
			gl.glVertex3d(v1.x, v1.y, v1.z);
		}
		gl.glEnd();

		//draw horizontal lines
		for(int i = 0; i < surfaceData.getVCount(); i+=6){
			gl.glBegin(GL.GL_LINE_STRIP);
			for(int j=0;j<surfaceData.getUCount();j++){
				Vector3D v1 = points[j][i];
				gl.glVertex3d(v1.x, v1.y, v1.z);
			}
			gl.glEnd();
		}
		gl.glBegin(GL.GL_LINE_STRIP);
		for(int j=0;j<surfaceData.getUCount();j++){
			Vector3D v1 = points[j][surfaceData.getVCount()-1];
			gl.glVertex3d(v1.x, v1.y, v1.z);
		}
		gl.glEnd();

		gl.glEnable(GL.GL_LIGHTING);

	}


}
