package gltest.surfaces;

import gltest.Vector3D;


/**
 * Defines a parametric Dini Surface surface.
 */
public class DiniSurface extends SurfaceParametric{
	

	protected void setDefaults() {
		uRes = 20;
		vRes = 12;
		umin = -4;
		umax =4;
		vmin = -Math.PI;
		vmax = Math.PI;
		setDefaultViewpoint(new Vector3D(10,-10,10));
		setDefaultWindow(-1.75,1.75,-1.75,1.75);
		
	}

	
	public Vector3D surfacePoint(double u, double v) {
	
	    double psi = Math.PI * Math.min(0.999,Math.max(0.35,0.001));
	    
	    double sinpsi = Math.sin(psi);
	    double cospsi = Math.cos(psi);
	    
	    double g = (u - cospsi * v) / sinpsi;
	    double s = Math.exp(g);
	    double r = (2 * sinpsi) / (s + 1 / s);
	    double t = r * (s - 1 / s) * 0.5;
	    
		double x = (u-t);
		double y = (r * Math.cos(v));
		double z = (r * Math.sin(v));
		return new Vector3D(x,y,z);
	}






}
